@@ -1,30 +1,58 @@
 use crate::opt::{ProcessCategory, SystemCategory};
 use crate::types::ProcessInfo;
-use crate::{CpuInfo, GpuInfo, Opts};
+use crate::{CpuInfo, GpuInfo, PhysicalCpuInfo};
 use plotters::prelude::*;
 use std::path::Path;
 
 pub fn consume<P: AsRef<Path>>(
-    output: &P,
-    opts: &Opts,
+    output: P,
     proc_category: &[ProcessCategory],
     sys_category: &[SystemCategory],
+    timestamps: &[chrono::DateTime<chrono::Local>],
     processes: &[ProcessInfo],
     cpu_info: &[CpuInfo],
     cpu_frequency_max: f32,
+    physical_cpu_info: &[PhysicalCpuInfo],
+    cpu_temperature_max: f32,
     gpu_info: &[GpuInfo],
 ) {
+    if timestamps.is_empty() {
+        return;
+    }
+
+    let timestamp_range = || timestamps[0].clone()..timestamps.last().cloned().unwrap();
+
+    let top_height = if !processes.is_empty() {
+        (processes.len() + 2) * 15
+    } else {
+        0
+    };
+
     let root = SVGBackend::new(
-        output,
+        &output,
         (
             1280,
-            720 * (proc_category.len() + sys_category.len()) as u32,
+            (top_height + 720 * (proc_category.len() + sys_category.len())) as u32,
         ),
     )
     .into_drawing_area();
     root.fill(&WHITE).unwrap();
 
-    let areas = root.split_evenly((proc_category.len() + sys_category.len(), 1));
+    let (top, bottom) = root.split_vertically(top_height as u32);
+    let default_font = ("sans-serif", 12).into_font();
+    let default_style: TextStyle = default_font.into();
+
+    for (i, p) in processes.into_iter().enumerate() {
+        let color = Palette99::pick(i).stroke_width(2).filled();
+        let i = i as i32;
+        let legend = PathElement::new(vec![(60, 23 + i * 15), (80, 23 + i * 15)], color);
+        top.draw(&legend).unwrap();
+        let label = format!("{}({}) - {}", p.name, p.pid, p.command);
+        top.draw_text(&label, &default_style, (90, 19 + i * 15))
+            .unwrap();
+    }
+
+    let areas = bottom.split_evenly((proc_category.len() + sys_category.len(), 1));
 
     // Draw process
     for idx_c in 0..proc_category.len() {
@@ -77,7 +105,7 @@ pub fn consume<P: AsRef<Path>>(
             .margin(10)
             .x_label_area_size(40)
             .y_label_area_size(50)
-            .build_cartesian_2d(0..(opts.count - 1), 0f32..max)
+            .build_cartesian_2d(timestamp_range(), 0f32..max)
             .unwrap();
 
         chart
@@ -90,7 +118,10 @@ pub fn consume<P: AsRef<Path>>(
             let color = Palette99::pick(idx).stroke_width(2).filled();
             chart
                 .draw_series(LineSeries::new(
-                    process.values[idx_c].clone().into_iter().enumerate(),
+                    timestamps
+                        .into_iter()
+                        .cloned()
+                        .zip(process.values[idx_c].iter().cloned()),
                     color.clone(),
                 ))
                 .unwrap()
@@ -110,7 +141,7 @@ pub fn consume<P: AsRef<Path>>(
             let color = Palette99::pick(processes.len()).stroke_width(2).filled();
             chart
                 .draw_series(LineSeries::new(
-                    total.into_iter().enumerate(),
+                    timestamps.into_iter().cloned().zip(total.into_iter()),
                     color.clone(),
                 ))
                 .unwrap()
@@ -136,11 +167,11 @@ pub fn consume<P: AsRef<Path>>(
         match c {
             SystemCategory::CPUFreq => {
                 chart = ChartBuilder::on(&area)
-                    .caption("CPU Frequency", ("sans-serif", 30).into_font())
+                    .caption("CPUs Frequency", ("sans-serif", 30).into_font())
                     .margin(10)
                     .x_label_area_size(40)
                     .y_label_area_size(50)
-                    .build_cartesian_2d(0..(opts.count - 1), 0f32..cpu_frequency_max)
+                    .build_cartesian_2d(timestamp_range(), 0f32..cpu_frequency_max)
                     .unwrap();
 
                 chart
@@ -153,11 +184,46 @@ pub fn consume<P: AsRef<Path>>(
                     let color = Palette99::pick(idx).stroke_width(2).filled();
                     chart
                         .draw_series(LineSeries::new(
-                            info.freq.clone().into_iter().enumerate(),
+                            timestamps
+                                .into_iter()
+                                .cloned()
+                                .zip(info.freq.iter().cloned()),
                             color.clone(),
                         ))
                         .unwrap()
-                        .label(format!("CPU{} / AVG({:.2}MHz)", idx, info.avg(),))
+                        .label(format!("CPU{} / AVG({:.2}MHz)", idx, info.freq_avg(),))
+                        .legend(move |(x, y)| {
+                            PathElement::new(vec![(x, y), (x + 20, y)], color.clone())
+                        });
+                }
+            }
+            SystemCategory::CPUTemp => {
+                chart = ChartBuilder::on(&area)
+                    .caption("CPUs Temperature", ("sans-serif", 30).into_font())
+                    .margin(10)
+                    .x_label_area_size(40)
+                    .y_label_area_size(50)
+                    .build_cartesian_2d(timestamp_range(), 0f32..cpu_temperature_max)
+                    .unwrap();
+
+                chart
+                    .configure_mesh()
+                    .y_label_formatter(&|y| format!("{}°C", y))
+                    .draw()
+                    .unwrap();
+
+                for (idx, info) in physical_cpu_info.iter().enumerate() {
+                    let color = Palette99::pick(idx).stroke_width(2).filled();
+                    chart
+                        .draw_series(LineSeries::new(
+                            timestamps
+                                .into_iter()
+                                .cloned()
+                                .zip(info.temp.iter().cloned()),
+                            color.clone(),
+                        ))
+                        .unwrap()
+                        .label(format!("CPU{} / AVG({:.2}°C)", idx, info.temp_avg(),))
                         .legend(move |(x, y)| {
                             PathElement::new(vec![(x, y), (x + 20, y)], color.clone())
                         });
@@ -169,7 +235,7 @@ pub fn consume<P: AsRef<Path>>(
                     .margin(10)
                     .x_label_area_size(40)
                     .y_label_area_size(50)
-                    .build_cartesian_2d(0..(opts.count - 1), 0.0f32..100.0f32)
+                    .build_cartesian_2d(timestamp_range(), 0.0f32..100.0f32)
                     .unwrap();
 
                 chart
@@ -182,7 +248,10 @@ pub fn consume<P: AsRef<Path>>(
                     let color = Palette99::pick(idx).stroke_width(2).filled();
                     chart
                         .draw_series(LineSeries::new(
-                            info.usage.clone().into_iter().enumerate(),
+                            timestamps
+                                .into_iter()
+                                .cloned()
+                                .zip(info.usage.iter().cloned()),
                             color.clone(),
                         ))
                         .unwrap()
