@@ -6,7 +6,7 @@ use std::path::PathBuf;
 use sysinfo::{ProcessExt, ProcessStatus, SystemExt};
 
 #[derive(Parser, Debug, Clone)]
-#[clap(version = "0.3.9")]
+#[clap(version = "0.4.3")]
 #[clap(about = "A command line tool to record process and system performance data.")]
 pub struct Opts {
     #[clap(short, long, multiple_values = true)]
@@ -131,6 +131,8 @@ pub enum Category {
     FPS,
     #[clap(name = "sys_cpu_freq")]
     SysCPUFreq,
+    #[clap(name = "sys_cpu_temp")]
+    SysCPUTemp,
     #[clap(name = "sys_gpu")]
     SysGPU,
 }
@@ -146,5 +148,6 @@ pub enum ProcessCategory {
 #[derive(Copy, Clone, PartialEq)]
 pub enum SystemCategory {
     CPUFreq,
+    CPUTemp,
     GPU,
 }
